/*
 * Copyright (c) 2023-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at:
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef DOCA_PIPE_GROUP_H
#define DOCA_PIPE_GROUP_H

#include <doca_flow.h>

#include "openvswitch/list.h"
#include "ovs-doca.h"

struct doca_pipe_group_ctx;
struct doca_pipe_group_mask_ctx;
struct netdev;

struct doca_pipe_group_ctx *
doca_pipe_group_ctx_ref(struct netdev *netdev, uint32_t group_id);
void
doca_pipe_group_ctx_unref(struct doca_pipe_group_ctx *group_ctx);

struct doca_flow_pipe *
doca_pipe_group_get_pipe(struct doca_pipe_group_ctx *group_ctx);

struct fixed_rule *
doca_pipe_group_get_miss_rule(struct doca_pipe_group_ctx *group_ctx);

void
doca_pipe_group_set_miss_flow_id(struct doca_pipe_group_ctx *group_ctx, uint32_t miss_flow_id);

uint32_t
doca_pipe_group_get_miss_flow_id(struct doca_pipe_group_ctx *group_ctx);

void doca_pipe_group_mask_ctx_unref(struct doca_pipe_group_mask_ctx *mask_ctx);

doca_error_t
doca_pipe_group_add_entry(struct netdev *netdev,
                          uint16_t pipe_queue,
                          uint32_t priority,
                          struct doca_pipe_group_ctx *group_ctx,
                          const struct doca_flow_match *match,
                          struct doca_flow_match *match_mask,
                          struct doca_flow_actions *actions,
                          struct doca_flow_actions *actions_mask,
                          const struct doca_flow_action_descs *action_descs,
                          const struct doca_flow_monitor *monitor,
                          const struct doca_flow_fwd *fwd,
                          struct doca_pipe_group_mask_ctx **mask_ctx,
                          struct doca_flow_pipe_entry **entry)
    OVS_EXCLUDED(mgmt_queue_lock);

void
doca_pipe_group_ctx_resize_end_list(struct ovs_list *list);

int
doca_pipe_group_ctx_add_miss(struct netdev *netdev,
                             struct doca_pipe_group_ctx *group_ctx);

bool
doca_pipe_group_is_ct_zone_group_id(uint32_t group);

#endif /* DOCA_PIPE_GROUP_H */
