/*
 * Copyright (c) 2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at:
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <config.h>
#include <stdlib.h>
#include <string.h>

#include "dp-packet.h"
#include "netdev-afxdp.h"
#include "netdev-dpdk.h"
#include "openvswitch/dynamic-string.h"
#include "util.h"

struct ds *
dp_packet_ds_put_hex(struct ds *s, struct dp_packet *b, uint32_t max_bytes)
{
    uint32_t mark, meta;
    uint32_t size, i;
    uint8_t *p;

    if (dp_packet_has_flow_mark(b, &mark)) {
        ds_put_format(s, "mark=%"PRIu32", ", mark);
    }
    if (dp_packet_get_meta(b, &meta)) {
        ds_put_format(s, "meta=0x%08"PRIx32", ", meta);
    }
    size = dp_packet_size(b);
    ds_put_format(s, "size=%"PRIu32". ", size);

    ds_put_cstr(s, "Raw(hex_bytes('");
    for (i = 0, p = dp_packet_data(b); i < MIN(size, max_bytes); i++, p++) {
        ds_put_format(s, "%02"PRIx8, *p);
    }
    ds_put_cstr(s, "'))");

    if (max_bytes < size) {
        ds_put_format(s, "/Raw('\\x00' * %"PRIu32")", size - max_bytes);
    }

    return s;
}
