/*
 * Copyright (c) 2023-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at:
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef DP_PACKET_EXT_H
#define DP_PACKET_EXT_H 1

#include "conntrack.h"
#include "conntrack-offload.h"
#include "mpsc-queue.h"
#include "openvswitch/netdev.h"
#include "openvswitch/types.h"

#ifdef  __cplusplus
extern "C" {
#endif

/* Types and macro definitions for e2e-cache. */

#define E2E_CACHE_MAX_TRACE   (10u)

enum e2e_cache_trace_flags {
    E2E_CACHE_TRACE_FLAG_TNL_POP     = 1 << 0,
    E2E_CACHE_TRACE_FLAG_OVERFLOW    = 1 << 1,
    E2E_CACHE_TRACE_FLAG_ABORT       = 1 << 2,
    E2E_CACHE_TRACE_FLAG_THROTTLED   = 1 << 3,
};

OVS_PACKED(
struct e2e_cache_trace_info {
    uint16_t num_elements;
    uint16_t e2e_trace_ct_ufids;
    odp_port_t orig_in_port;
    OVS_ALIGNED_VAR(sizeof(void *)) ovs_u128 ufids[E2E_CACHE_MAX_TRACE];
});

/* The address of the field 'node' will be used for list insertion,
 * and an explicit alignment is necessary for the compiler to understand
 * this address use is fine.
 */
OVS_PACKED(
struct e2e_cache_trace_message {
    OVS_ALIGNED_VAR(sizeof(void *)) struct mpsc_queue_node node;
    void *dp;
    long long int timestamp;
    uint32_t num_elements;
    uint32_t pad;
    OVS_ALIGNED_VAR((sizeof(void *))) struct e2e_cache_trace_info data[0];
});

#define DP_PACKET_E2E \
BUILD_ASSERT_DECL(E2E_CACHE_MAX_TRACE <= 16); \
    uint32_t   e2e_trace_size;                \
    uint16_t   e2e_trace_flags;               \
    uint16_t   e2e_trace_ct_ufids;            \
    ovs_u128   e2e_trace[E2E_CACHE_MAX_TRACE];

#ifdef DPDK_NETDEV

#else /* DPDK_NETDEV */

#endif /* DPDK_NETDEV */

struct ds *
dp_packet_ds_put_hex(struct ds *s, struct dp_packet *b, uint32_t max_bytes);

#ifdef  __cplusplus
}
#endif

#endif /* dp-packet-ext.h */
