/*
 * Copyright (c) 2023-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at:
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef DPDK_OFFLOAD_DOCA
#define DPDK_OFFLOAD_DOCA

#include "dpdk-offload-provider.h"
#include "ovs-doca.h"

struct netdev;
struct doca_offload_esw_ctx;

struct destroy_pipe_elem {
    struct doca_flow_pipe *pipe;
    struct ovs_list node;
};

void
doca_offload_fixed_rule_uninit(struct doca_offload_esw_ctx *esw,
                               struct netdev *netdev, struct fixed_rule *fr);

/* Poll the queue 'qid' for completion. If 'sync' is true, wait until
 * all entries in the queue have been completed and all pipes have
 * completed their resizing. */
doca_error_t
doca_offload_complete_queue(struct netdev *netdev, unsigned int qid,
                            bool sync);

struct doca_offload_esw_ctx *
doca_offload_esw_ctx_get(const struct netdev *netdev);

void
doca_offload_esw_ctx_pipe_resizing_ref(struct doca_offload_esw_ctx *esw_ctx);

void
doca_offload_esw_ctx_pipe_resizing_unref(struct doca_offload_esw_ctx *esw_ctx);

void
doca_offload_esw_ctx_resized_list_add(struct doca_offload_esw_ctx *esw_ctx,
                                      struct ovs_list *resized_list_node);

struct doca_flow_pipe_entry *
dpdk_offload_sample_entry_get(const struct netdev *netdev);

doca_error_t
doca_offload_add_entry(unsigned int qid,
                       struct doca_flow_pipe *pipe,
                       const struct doca_flow_match *match,
                       const struct doca_flow_actions *actions,
                       const struct doca_flow_monitor *monitor,
                       const struct doca_flow_fwd *fwd,
                       uint32_t flags,
                       struct doca_offload_esw_ctx *esw,
                       struct doca_flow_pipe_entry **pentry);

doca_error_t
doca_offload_remove_entry(struct doca_offload_esw_ctx *esw,
                          unsigned int qid, uint32_t flags,
                          struct doca_flow_pipe_entry *entry)
    OVS_EXCLUDED(mgmt_queue_lock);

struct ovs_list *
doca_offload_destroy_list_get(struct doca_offload_esw_ctx *esw_ctx);

struct ovs_rwlock *
doca_offload_rwlock_get(const struct netdev *netdev);

void
doca_offload_entry_process(struct doca_flow_pipe_entry *entry,
                           uint16_t qid,
                           enum doca_flow_entry_status status,
                           enum doca_flow_entry_op op,
                           void *aux);

void
doca_offload_set_reg_template(struct doca_flow_meta *dmeta, enum dpdk_reg_id id);

void
doca_offload_set_reg_mask(struct doca_flow_meta *dmeta, enum dpdk_reg_id id);

int
doca_offload_query_entry(struct ovs_rwlock *port_rwlock,
                         struct doca_flow_pipe_entry *entry,
                         struct doca_flow_resource_query *query_stats);

#endif /* DPDK_OFFLOAD_DOCA */
